const GamePlay = function(opts={}) {

  const NO_OF_PLAYERS = 2,
        NO_OF_STARTING_CARDS = 2,
        Deck = require("./deck.js").Deck,
        Player = require("./player.js").Player,
        Hand = require("pokersolver").Hand;
        _ = require("underscore");

  this.game_status = opts["game_status"] ? opts["game_status"] : "start";
  this.deck = opts["deck"] ? opts["deck"] : new Deck().shuffle();
  this.table_cards = opts["table_cards"] ? opts["table_cards"] : [];
  this.players = [new Player(opts["player_1"]), new Player(opts["player_2"])];
  this.winners = [];

  this.deal = function(){
    let self = this;
    //TODO instead of return, raise errors
    if(self.game_status !== "start"){ return; }
    if(self.players[0]["id"] == undefined && self.players[1]["id"] == undefined) { return;}
    let i = 0;
    for(i; i < NO_OF_STARTING_CARDS; i++){
      self.players.forEach(function(player, index){
        self.players[index].cards.push(self.deck.shift())
      });
    }
    this.set_players_status("ready");
    this.game_status = "deal";
    return this;
  };

  this.flop = function(){
    let self = this;
    if(self.game_status !== "deal"){ return; }
    let discard = self.deck.shift();
    let i = 0;
    for(i; i < 3; i++){
      self.table_cards.push(self.deck.shift());
    }
    this.set_players_status("ready");
    this.game_status = "flop";
    return this;
  };

  this.turn = function(){
    if(this.game_status !== "flop"){ return; }
    let discard = this.deck.shift();
    this.table_cards.push(this.deck.shift());
    this.set_players_status("ready");
    this.game_status = "turn";
    return this;
  };

  this.river = function(){
    if(this.game_status !== "turn"){ return; }
    let discard = this.deck.shift();
    this.table_cards.push(this.deck.shift());
    this.set_players_status("ready");
    this.game_status = "river";
    return this;
  };

  this.end = function(){
    let self = this;
    if(self.game_status !== "river"){ return; }
    let player_1_hand = Hand.solve(self.players[0].cards.concat(self.table_cards));
    let player_2_hand = Hand.solve(self.players[1].cards.concat(self.table_cards));
    let winning_hands = Hand.winners([player_1_hand, player_2_hand]);
    if(winning_hands.length ==1){
      if(winning_hands[0] == player_1_hand){
        self.winners.push({"id": self.players[0].id, "cards": this.parse_hand(winning_hands), "hand_type": this.hand_type(player_1_hand), "description": player_1_hand.descr, "winner": 1, "rank": player_1_hand.rank});
      } else if(winning_hands[0] == player_2_hand){
        self.winners.push({"id": self.players[1].id, "cards": this.parse_hand(winning_hands), "hand_type": this.hand_type(player_2_hand), "description": player_2_hand.descr, "winner": 2, "rank": player_2_hand.rank});
      }
    } else {
      self.winners.push({"id": self.players[0].id, "cards": this.parse_hand(winning_hands), "hand_type": this.hand_type(player_1_hand), "description": player_1_hand.descr, "winner": "draw", "rank": player_1_hand.rank});
      self.winners.push({"id": self.players[1].id, "cards": this.parse_hand(winning_hands), "hand_type": this.hand_type(player_2_hand), "description": player_2_hand.descr, "winner": "draw", "rank": player_2_hand.rank});
    }

    this.game_status = "end";
    return this;
  };

  this.discard = function(player_id, card_index){
    let player = this.get_player(player_id);
    if(player && this.player_can_discard(player)){
      player.cards[card_index] = this.deck.shift();
      player.status = "discarded"
      return player.cards[card_index];
    } else {
      return false;
    }
  };

  this.burn = function(player_id){
    let player = this.get_player(player_id);
    if(player && this.player_can_burn(player)){
      player.cards[0] = this.deck.shift();
      player.cards[1] = this.deck.shift();
      player.status = "discarded"
      return player.cards;
    } else {
      return false;
    }
  };

  this.player_waiting = function(player_id){
   player = this.get_player(player_id);
   player.status = "waiting";
  };

  this.to_json = function(){
    return {
      "player_1": this.players[0],
      "player_2": this.players[1],
      "game_status": this.game_status,
      "deck": this.deck,
      "table_cards": this.table_cards,
      "winners": this.winners
    }
  };

  // PRIVATE

  this.parse_hand = function(winning_hand){
    hand = winning_hand.toString().split(", ");
    hand = _.map(hand, function(card){
      return card.replace("10","T");
    });
    return hand.slice(0,5);
  };

  this.hand_type = function(hand){
    if(hand.descr.indexOf("Royal Flush") !== -1) {
      return "Royal Flush";
    } else {
      return hand.name
    }
  };

  this.get_player = function(player_id){
    return _.find(this.players, function(player){ return player.id == player_id});
  };

  this.player_can_discard = function(player){
    if(player.status != "ready") { return false; }
    let val = false;
    switch(this.game_status) {
      case "deal":
        val = true;
        break;
      case "flop":
        val = true;
        break;
      case "turn":
        val = true;
        break;
      default: 
        val = false;
    }
    return val;
  };

  this.player_can_burn = function(player){
    if(player.status != "ready") { return false; }
    return this.game_status == "river" ? true : false;
  };

  this.set_players_status = function(status) {
    _.each(this.players, function(player){
      player.status = status;
    });
  };

};

exports.GamePlay = GamePlay;
